{---------------------------------------------------------------------------}
{- Don't use these procedures from here... ---------------------------------}
{- Better try "uses ScrnVGA, ScrnPal, Misc" --------------------------------}
{---------------------------------------------------------------------------}

function DetectVGA: Boolean; assembler;
asm
  mov     AX, 1A00h
  int     10h
  cmp     AL, 1Ah
  je      @FoundVGA
  mov     AX, FALSE
  jmp     @Done
@FoundVGA:
  mov     AX, TRUE
@Done:
end;

{---------------------------------------------------------------------------}

function GetVideoMode: Byte;
begin
  GetVideoMode := Mem[$0000:$0449];
end;

{---------------------------------------------------------------------------}

procedure Delay(ms: Word); assembler;
asm
  mov     AX, 1000
  mul     ms
  mov     CX, DX
  mov     DX, AX
  mov     AH, 86h
  int     15h
end;

{---------------------------------------------------------------------------}

procedure Retrace;
begin
  while (Port[$3DA] and 8) <> 8 do;
  while (Port[$3DA] and 8) = 8 do;
end;

{---------------------------------------------------------------------------}

procedure CLS(Clr: Byte); assembler;
asm
  mov     AX, $A000
  mov     ES, AX
  xor     DI, DI
  mov     AH, Clr
  mov     AL, Clr
  mov     CX, 32000
  cld
  rep     stosw
end;

{---------------------------------------------------------------------------}

function GetPixel(X, Y: Integer): Byte; assembler;
asm
  cmp     X, 0
  jl      @Done
  cmp     Y, 0
  jl      @Done
  cmp     X, 319
  jg      @Done
  cmp     Y, 199
  jg      @Done
  mov     AX, $A000
  mov     ES, AX
  mov     AX, [Y]
  mov     BX, 320
  mul     BX
  mov     DI, [X]
  add     DI, AX
  mov     AL, ES:[DI]
@Done:
end;

{---------------------------------------------------------------------------}

procedure PutPixel(X, Y: Integer; Clr: Byte); assembler;
asm
  cmp     X, 0
  jl      @Done
  cmp     Y, 0
  jl      @Done
  cmp     X, 319
  jg      @Done
  cmp     Y, 199
  jg      @Done
  mov     AX, $A000
  mov     ES, AX
  mov     BX, [X]
  mov     DX, [Y]
  mov     DI, BX
  mov     BX, DX
  shl     DX, 0008h
  shl     BX, 0006h
  add     DX, BX
  add     DI, DX
  mov     AL, [Clr]
  cld
  stosb
@Done:
end;

{---------------------------------------------------------------------------}

procedure Line(X1, Y1, X2, Y2: Integer; Clr: Byte);
var I, DeltaX, DeltaY,
    NumPixels,
    D, DInc1, DInc2,
    X, XInc1, XInc2,
    Y, YInc1, YInc2   : Integer;
begin
  DeltaX := Abs(X2 - X1);
  DeltaY := Abs(Y2 - Y1);
  if DeltaX >= DeltaY then begin
    NumPixels := DeltaX + 1;
    D := (2 * DeltaY) - DeltaX;
    DInc1 := DeltaY shl 1;
    DInc2 := (DeltaY - DeltaX) shl 1;
    XInc1 := 1;
    XInc2 := 1;
    YInc1 := 0;
    YInc2 := 1;
  end else begin
    NumPixels := DeltaY + 1;
    D := (2 * DeltaX) - DeltaY;
    DInc1 := DeltaX shl 1;
    DInc2 := (DeltaX - DeltaY) shl 1;
    XInc1 := 0;
    XInc2 := 1;
    YInc1 := 1;
    YInc2 := 1;
  end;
  if X1 > X2 then begin
    XInc1 := -XInc1;
    XInc2 := -XInc2;
  end;
  if Y1 > Y2 then begin
    YInc1 := -YInc1;
    YInc2 := -YInc2;
  end;
  X := X1;
  Y := Y1;
  for I := 1 to NumPixels do begin
    PutPixel(X, Y, Clr);
    if D < 0 then begin
      D := D + DInc1;
      X := X + XInc1;
      Y := Y + YInc1;
    end else begin
      D := D + DInc2;
      X := X + XInc2;
      Y := Y + YInc2;
    end;
  end;
end;

{---------------------------------------------------------------------------}

procedure HLine(X1, X2, Y: Integer; Clr: Byte);
var I: Integer;
begin
  if X1 > X2 then begin
    I := X1;
    X1 := X2;
    X2 := I;
  end;
  for I := X1 to X2 do PutPixel(I, Y, Clr);
end;

{---------------------------------------------------------------------------}

procedure WriteTxt(X, Y: Integer; Txt: string; Clr: Byte);
var T1, T2, T3: Byte;
begin
  if X > -32760 then X := X - 8;
  for T1 := 1 to Length(Txt) do
    for T2 := 0 to 7 do
      for T3 := 0 to 7 do
        if (Mem[FSeg:FOfs + T2 + (Ord(Txt[T1]) * 8)] shl T3)
          and 128 <> 0 then
            PutPixel(X + T3 + (T1 * 8), Y + T2, Clr);
end;

{---------------------------------------------------------------------------}

procedure WriteCnt(Row: Byte; Txt: string; Clr: Byte);
begin
  WriteTxt(159 - Round((Length(Txt) / 2) * 8), Row, Txt, Clr);
end;

{---------------------------------------------------------------------------}

procedure SetRGB(Clr, R, G, B: Byte);
begin
  Port[$3C8] := Clr;
  Port[$3C9] := R;
  Port[$3C9] := G;
  Port[$3C9] := B;
end;

{---------------------------------------------------------------------------}

procedure GetRGB(Clr: Byte; var R, G, B: Byte);
begin
  Port[$3C7] := Clr;
  R := Port[$3C9];
  G := Port[$3C9];
  B := Port[$3C9];
end;

{---------------------------------------------------------------------------}

procedure RotatePal(MinC, MaxC: Byte);
var Tmp,
    Tmp2 : array[1..3] of Byte;
begin
  GetRGB(MaxC, Tmp[1], Tmp[2], Tmp[3]);
  for BTmp := MaxC - 1 downto MinC do begin
    GetRGB(BTmp, Tmp2[1], Tmp2[2], Tmp2[3]);
    SetRGB(BTmp + 1, Tmp2[1], Tmp2[2], Tmp2[3]);
  end;
  SetRGB(MinC, Tmp[1], Tmp[2], Tmp[3]);
end;

{---------------------------------------------------------------------------}

procedure FadeMax;
var Tmp  : Byte;
    TmpB : array[1..3] of Byte;
begin
  for BTmp := 0 to 63 do begin
    Retrace;
    for Tmp := 0 to 255 do begin
      GetRGB(Tmp, TmpB[1], TmpB[2], TmpB[3]);
      if TmpB[1] < 63 then Inc(TmpB[1]);
      if TmpB[2] < 63 then Inc(TmpB[2]);
      if TmpB[3] < 63 then Inc(TmpB[3]);
      SetRGB(Tmp, TmpB[1], TmpB[2], TmpB[3]);
    end;
  end;
end;

{---------------------------------------------------------------------------}

procedure FadeDown;
var Tmp  : Byte;
    TmpB : array[1..3] of Byte;
begin
  for BTmp := 0 to 63 do begin
    Retrace;
    for Tmp := 0 to 255 do begin
      GetRGB(Tmp, TmpB[1], TmpB[2], TmpB[3]);
      if TmpB[1] > 0 then Dec(TmpB[1]);
      if TmpB[2] > 0 then Dec(TmpB[2]);
      if TmpB[3] > 0 then Dec(TmpB[3]);
      SetRGB(Tmp, TmpB[1], TmpB[2], TmpB[3]);
    end;
  end;
end;